@@ -2,6 +2,7 @@ from itertools import izip
 import types
 import numpy as np
 
+from pandas.core.base import PandasObject
 from pandas.core.categorical import Categorical
 from pandas.core.frame import DataFrame
 from pandas.core.generic import NDFrame
@@ -13,6 +14,7 @@ from pandas.util.decorators import cache_readonly, Appender
 from pandas.util.compat import OrderedDict
 import pandas.core.algorithms as algos
 import pandas.core.common as com
+from pandas.core.common import _possibly_downcast_to_dtype, notnull
 
 import pandas.lib as lib
 import pandas.algos as _algos
@@ -25,7 +27,7 @@ Parameters
 arg : function or dict
     Function to use for aggregating groups. If a function, must either
     work when passed a DataFrame or when passed to DataFrame.apply. If
-    pass a dict, the keys must be DataFrame column names
+    passed a dict, the keys must be DataFrame column names.
 
 Notes
 -----
@@ -57,6 +59,8 @@ def _groupby_function(name, alias, npfunc, numeric_only=True,
     def f(self):
         try:
             return self._cython_agg_general(alias, numeric_only=numeric_only)
+        except AssertionError as e:
+            raise SpecificationError(str(e))
         except Exception:
             result = self.aggregate(lambda x: npfunc(x, axis=self.axis))
             if _convert:
@@ -72,7 +76,7 @@ def _groupby_function(name, alias, npfunc, numeric_only=True,
 def _first_compat(x, axis=0):
     def _first(x):
         x = np.asarray(x)
-        x = x[com.notnull(x)]
+        x = x[notnull(x)]
         if len(x) == 0:
             return np.nan
         return x[0]
@@ -86,7 +90,7 @@ def _first_compat(x, axis=0):
 def _last_compat(x, axis=0):
     def _last(x):
         x = np.asarray(x)
-        x = x[com.notnull(x)]
+        x = x[notnull(x)]
         if len(x) == 0:
             return np.nan
         return x[-1]
@@ -97,7 +101,7 @@ def _last_compat(x, axis=0):
         return _last(x)
 
 
-class GroupBy(object):
+class GroupBy(PandasObject):
     """
     Class for grouping and aggregating relational data. See aggregate,
     transform, and apply functions on this object.
@@ -166,7 +170,7 @@ class GroupBy(object):
 
     def __init__(self, obj, keys=None, axis=0, level=None,
                  grouper=None, exclusions=None, selection=None, as_index=True,
-                 sort=True, group_keys=True):
+                 sort=True, group_keys=True, squeeze=False):
         self._selection = selection
 
         if isinstance(obj, NDFrame):
@@ -186,6 +190,7 @@ class GroupBy(object):
         self.keys = keys
         self.sort = sort
         self.group_keys = group_keys
+        self.squeeze = squeeze
 
         if grouper is None:
             grouper, exclusions = _get_grouper(obj, keys, axis=axis,
@@ -197,6 +202,10 @@ class GroupBy(object):
     def __len__(self):
         return len(self.indices)
 
+    def __unicode__(self):
+        # TODO: Better unicode/repr for GroupBy object
+        return object.__repr__(self)
+
     @property
     def groups(self):
         return self.grouper.groups
@@ -348,7 +357,7 @@ class GroupBy(object):
         """
         try:
             return self._cython_agg_general('mean')
-        except DataError:
+        except GroupByError:
             raise
         except Exception:  # pragma: no cover
             f = lambda x: x.mean(axis=self.axis)
@@ -362,7 +371,7 @@ class GroupBy(object):
         """
         try:
             return self._cython_agg_general('median')
-        except DataError:
+        except GroupByError:
             raise
         except Exception:  # pragma: no cover
             f = lambda x: x.median(axis=self.axis)
@@ -418,13 +427,26 @@ class GroupBy(object):
 
     def nth(self, n):
         def picker(arr):
-            arr = arr[com.notnull(arr)]
+            arr = arr[notnull(arr)]
             if len(arr) >= n + 1:
                 return arr.iget(n)
             else:
                 return np.nan
         return self.agg(picker)
 
+    def _try_cast(self, result, obj):
+        """ try to cast the result to our obj original type,
+        we may have roundtripped thru object in the mean-time """
+        if obj.ndim > 1:
+            dtype = obj.values.dtype
+        else:
+            dtype = obj.dtype
+
+        if not np.isscalar(result):
+            result = _possibly_downcast_to_dtype(result, dtype)
+
+        return result
+
     def _cython_agg_general(self, how, numeric_only=True):
         output = {}
         for name, obj in self._iterate_slices():
@@ -432,8 +454,11 @@ class GroupBy(object):
             if numeric_only and not is_numeric:
                 continue
 
-            result, names = self.grouper.aggregate(obj.values, how)
-            output[name] = result
+            try:
+                result, names = self.grouper.aggregate(obj.values, how)
+            except AssertionError as e:
+                raise GroupByError(str(e))
+            output[name] = self._try_cast(result, obj)
 
         if len(output) == 0:
             raise DataError('No numeric types to aggregate')
@@ -449,7 +474,7 @@ class GroupBy(object):
         for name, obj in self._iterate_slices():
             try:
                 result, counts = self.grouper.agg_series(obj, f)
-                output[name] = result
+                output[name] = self._try_cast(result, obj)
             except TypeError:
                 continue
 
@@ -457,9 +482,16 @@ class GroupBy(object):
             return self._python_apply_general(f)
 
         if self.grouper._filter_empty_groups:
+
             mask = counts.ravel() > 0
             for name, result in output.iteritems():
-                output[name] = result[mask]
+
+                # since we are masking, make sure that we have a float object
+                values = result
+                if _is_numeric_dtype(values.dtype):
+                    values = com.ensure_float(values)
+
+                output[name] = self._try_cast(values[mask],result)
 
         return self._wrap_aggregated_output(output)
 
@@ -584,7 +616,9 @@ class Grouper(object):
             try:
                 values, mutated = splitter.fast_apply(f, group_keys)
                 return group_keys, values, mutated
-            except lib.InvalidApply:
+            except (Exception), detail:
+                # we detect a mutatation of some kind
+                # so take slow path
                 pass
 
         result_values = []
@@ -708,21 +742,16 @@ class Grouper(object):
     # Aggregation functions
 
     _cython_functions = {
-        'add': _algos.group_add,
-        'prod': _algos.group_prod,
-        'min': _algos.group_min,
-        'max': _algos.group_max,
-        'mean': _algos.group_mean,
-        'median': _algos.group_median,
-        'var': _algos.group_var,
-        'std': _algos.group_var,
-        'first': lambda a, b, c, d: _algos.group_nth(a, b, c, d, 1),
-        'last': _algos.group_last
-    }
-
-    _cython_object_functions = {
-        'first': lambda a, b, c, d: _algos.group_nth_object(a, b, c, d, 1),
-        'last': _algos.group_last_object
+        'add'  : 'group_add',
+        'prod' : 'group_prod',
+        'min'  : 'group_min',
+        'max'  : 'group_max',
+        'mean' : 'group_mean',
+        'median': dict(name = 'group_median'),
+        'var'  : 'group_var',
+        'std'  : 'group_var',
+        'first': dict(name = 'group_nth', f = lambda func, a, b, c, d: func(a, b, c, d, 1)),
+        'last' : 'group_last',
     }
 
     _cython_transforms = {
@@ -737,6 +766,40 @@ class Grouper(object):
 
     _filter_empty_groups = True
 
+    def _get_aggregate_function(self, how, values):
+
+        dtype_str = values.dtype.name
+        def get_func(fname):
+
+            # find the function, or use the object function, or return a generic
+            for dt in [dtype_str,'object']:
+                f = getattr(_algos,"%s_%s" % (fname,dtype_str),None)
+                if f is not None:
+                    return f
+            return getattr(_algos,fname,None)
+
+        ftype = self._cython_functions[how]
+
+        if isinstance(ftype,dict):
+            func = afunc = get_func(ftype['name'])
+
+            # a sub-function
+            f = ftype.get('f')
+            if f is not None:
+
+                def wrapper(*args, **kwargs):
+                    return f(afunc, *args, **kwargs)
+
+                # need to curry our sub-function
+                func = wrapper
+
+        else:
+            func = get_func(ftype)
+
+        if func is None:
+            raise NotImplementedError("function is not implemented for this dtype: [how->%s,dtype->%s]" % (how,dtype_str))
+        return func, dtype_str
+
     def aggregate(self, values, how, axis=0):
 
         arity = self._cython_arity.get(how, 1)
@@ -796,12 +859,8 @@ class Grouper(object):
         return result, names
 
     def _aggregate(self, result, counts, values, how, is_numeric):
-        if not is_numeric:
-            agg_func = self._cython_object_functions[how]
-        else:
-            agg_func = self._cython_functions[how]
-
-        trans_func = self._cython_transforms.get(how, lambda x: x)
+        agg_func,dtype  = self._get_aggregate_function(how, values)
+        trans_func      = self._cython_transforms.get(how, lambda x: x)
 
         comp_ids, _, ngroups = self.group_info
         if values.ndim > 3:
@@ -809,8 +868,9 @@ class Grouper(object):
             raise NotImplementedError
         elif values.ndim > 2:
             for i, chunk in enumerate(values.transpose(2, 0, 1)):
-                agg_func(result[:, :, i], counts, chunk.squeeze(),
-                         comp_ids)
+
+                chunk = chunk.squeeze()
+                agg_func(result[:, :, i], counts, chunk, comp_ids)
         else:
             agg_func(result, counts, values, comp_ids)
 
@@ -834,7 +894,7 @@ class Grouper(object):
         dummy = obj[:0].copy()
         indexer = _algos.groupsort_indexer(group_index, ngroups)[0]
         obj = obj.take(indexer)
-        group_index = com.ndtake(group_index, indexer)
+        group_index = com.take_nd(group_index, indexer, allow_fill=False)
         grouper = lib.SeriesGrouper(obj, func, group_index, ngroups,
                                     dummy)
         result, counts = grouper.get_result()
@@ -948,8 +1008,8 @@ class BinGrouper(Grouper):
                 yield label, data[start:edge]
                 start = edge
 
-            if edge < len(data):
-                yield self.binlabels[-1], data[edge:]
+            if start < len(data):
+                yield self.binlabels[-1], data[start:]
         else:
             start = 0
             for edge, label in izip(self.bins, self.binlabels):
@@ -958,8 +1018,8 @@ class BinGrouper(Grouper):
                 start = edge
 
             n = len(data.axes[axis])
-            if edge < n:
-                inds = range(edge, n)
+            if start < n:
+                inds = range(start, n)
                 yield self.binlabels[-1], data.take(inds, axis=axis)
 
     def apply(self, f, data, axis=0, keep_internal=False):
@@ -1000,21 +1060,16 @@ class BinGrouper(Grouper):
     # cython aggregation
 
     _cython_functions = {
-        'add': _algos.group_add_bin,
-        'prod': _algos.group_prod_bin,
-        'mean': _algos.group_mean_bin,
-        'min': _algos.group_min_bin,
-        'max': _algos.group_max_bin,
-        'var': _algos.group_var_bin,
-        'std': _algos.group_var_bin,
-        'ohlc': _algos.group_ohlc,
-        'first': lambda a, b, c, d: _algos.group_nth_bin(a, b, c, d, 1),
-        'last': _algos.group_last_bin
-    }
-
-    _cython_object_functions = {
-        'first': lambda a, b, c, d: _algos.group_nth_bin_object(a, b, c, d, 1),
-        'last': _algos.group_last_bin_object
+        'add'  : 'group_add_bin',
+        'prod' : 'group_prod_bin',
+        'mean' : 'group_mean_bin',
+        'min'  : 'group_min_bin',
+        'max'  : 'group_max_bin',
+        'var'  : 'group_var_bin',
+        'std'  : 'group_var_bin',
+        'ohlc' : 'group_ohlc',
+        'first': dict(name = 'group_nth_bin', f = lambda func, a, b, c, d: func(a, b, c, d, 1)),
+        'last' : 'group_last_bin',
     }
 
     _name_functions = {
@@ -1024,11 +1079,9 @@ class BinGrouper(Grouper):
     _filter_empty_groups = True
 
     def _aggregate(self, result, counts, values, how, is_numeric=True):
-        fdict = self._cython_functions
-        if not is_numeric:
-            fdict = self._cython_object_functions
-        agg_func = fdict[how]
-        trans_func = self._cython_transforms.get(how, lambda x: x)
+
+        agg_func,dtype = self._get_aggregate_function(how, values)
+        trans_func     = self._cython_transforms.get(how, lambda x: x)
 
         if values.ndim > 3:
             # punting for now
@@ -1144,6 +1197,13 @@ class Grouping(object):
             # no level passed
             if not isinstance(self.grouper, np.ndarray):
                 self.grouper = self.index.map(self.grouper)
+                if not (hasattr(self.grouper,"__len__") and \
+                   len(self.grouper) == len(self.index)):
+                    errmsg = "Grouper result violates len(labels) == len(data)\n"
+                    errmsg += "result: %s" % com.pprint_thing(self.grouper)
+                    self.grouper = None # Try for sanity
+                    raise AssertionError(errmsg)
+
 
     def __repr__(self):
         return 'Grouping(%s)' % self.name
@@ -1197,11 +1257,14 @@ def _get_grouper(obj, key=None, axis=0, level=None, sort=True):
 
     if level is not None:
         if not isinstance(group_axis, MultiIndex):
-            if level > 0:
+            if isinstance(level, basestring):
+                if obj.index.name != level:
+                    raise ValueError('level name %s is not the name of the index' % level)
+            elif level > 0:
                 raise ValueError('level > 0 only valid with MultiIndex')
-            else:
-                level = None
-                key = group_axis
+            
+            level = None
+            key = group_axis
 
     if isinstance(key, CustomGrouper):
         gpr = key.get_grouper(obj)
@@ -1254,6 +1317,11 @@ def _get_grouper(obj, key=None, axis=0, level=None, sort=True):
             exclusions.append(gpr)
             name = gpr
             gpr = obj[gpr]
+
+        if (isinstance(gpr,Categorical) and len(gpr) != len(obj)):
+            errmsg = "Categorical grouper must have len(grouper) == len(data)"
+            raise AssertionError(errmsg)
+
         ping = Grouping(group_axis, gpr, name=name, level=level, sort=sort)
         groupings.append(ping)
 
@@ -1304,8 +1372,8 @@ class SeriesGroupBy(GroupBy):
         -----
         agg is an alias for aggregate. Use it.
 
-        Example
-        -------
+        Examples
+        --------
         >>> series
         bar    1.0
         baz    2.0
@@ -1360,7 +1428,7 @@ class SeriesGroupBy(GroupBy):
             ret = Series(result, index=index)
 
         if not self.as_index:  # pragma: no cover
-            print 'Warning, ignoring as_index=True'
+            print ('Warning, ignoring as_index=True')
 
         return ret
 
@@ -1439,7 +1507,7 @@ class SeriesGroupBy(GroupBy):
             output = func(group, *args, **kwargs)
             if isinstance(output, np.ndarray):
                 raise Exception('Must produce aggregated value')
-            result[name] = output
+            result[name] = self._try_cast(output, group)
 
         return result
 
@@ -1453,8 +1521,8 @@ class SeriesGroupBy(GroupBy):
         func : function
             To apply to each group. Should return a Series with the same index
 
-        Example
-        -------
+        Examples
+        --------
         >>> grouped.transform(lambda x: (x - x.mean()) / x.std())
 
         Returns
@@ -1462,6 +1530,9 @@ class SeriesGroupBy(GroupBy):
         transformed : Series
         """
         result = self.obj.copy()
+        if hasattr(result,'values'):
+            result = result.values
+        dtype = result.dtype
 
         if isinstance(func, basestring):
             wrapper = lambda x: getattr(x, func)(*args, **kwargs)
@@ -1469,14 +1540,58 @@ class SeriesGroupBy(GroupBy):
             wrapper = lambda x: func(x, *args, **kwargs)
 
         for name, group in self:
+
+            group = com.ensure_float(group)
             object.__setattr__(group, 'name', name)
             res = wrapper(group)
-            # result[group.index] = res
             indexer = self.obj.index.get_indexer(group.index)
-            np.put(result, indexer, res)
+            if hasattr(res,'values'):
+                res = res.values
 
-        return result
+            # need to do a safe put here, as the dtype may be different
+            # this needs to be an ndarray
+            result,_ = com._maybe_upcast_indexer(result, indexer, res)
 
+        # downcast if we can (and need)
+        result = _possibly_downcast_to_dtype(result, dtype)
+        return self.obj.__class__(result,index=self.obj.index,name=self.obj.name)
+
+    def filter(self, func, dropna=True, *args, **kwargs):
+        """
+        Return a copy of a Series excluding elements from groups that
+        do not satisfy the boolean criterion specified by func.
+
+        Parameters
+        ----------
+        func : function
+            To apply to each group. Should return True or False.
+        dropna : Drop groups that do not pass the filter. True by default;
+            if False, groups that evaluate False are filled with NaNs.
+
+        Example
+        -------
+        >>> grouped.filter(lambda x: x.mean() > 0)
+
+        Returns
+        -------
+        filtered : Series
+        """
+        if isinstance(func, basestring):
+            wrapper = lambda x: getattr(x, func)(*args, **kwargs)
+        else:
+            wrapper = lambda x: func(x, *args, **kwargs)
+
+        indexers = [self.obj.index.get_indexer(group.index) \
+                    if wrapper(group) else [] for _ , group in self]
+
+        if len(indexers) == 0:
+            filtered = self.obj.take([]) # because np.concatenate would fail
+        else:
+            filtered = self.obj.take(np.concatenate(indexers))
+        if dropna:
+            return filtered
+        else:
+            return filtered.reindex(self.obj.index) # Fill with NaNs.
 
 class NDFrameGroupBy(GroupBy):
 
@@ -1538,6 +1653,10 @@ class NDFrameGroupBy(GroupBy):
                 values = com.ensure_float(values)
 
             result, _ = self.grouper.aggregate(values, how, axis=agg_axis)
+
+            # see if we can cast the block back to the original dtype
+            result = block._try_cast_result(result)
+
             newb = make_block(result, block.items, block.ref_items)
             new_blocks.append(newb)
 
@@ -1579,7 +1698,7 @@ class NDFrameGroupBy(GroupBy):
             if self.axis != 0:  # pragma: no cover
                 raise ValueError('Can only pass dict with axis=0')
 
-            obj = self._obj_with_exclusions
+            obj = self.obj
 
             if any(isinstance(x, (list, tuple, dict)) for x in arg.values()):
                 new_arg = OrderedDict()
@@ -1630,7 +1749,9 @@ class NDFrameGroupBy(GroupBy):
                 zipped = zip(result.index.levels, result.index.labels,
                              result.index.names)
                 for i, (lev, lab, name) in enumerate(zipped):
-                    result.insert(i, name, com.ndtake(lev.values, lab))
+                    result.insert(i, name,
+                                  com.take_nd(lev.values, lab,
+                                              allow_fill=False))
                 result = result.consolidate()
             else:
                 values = result.index.values
@@ -1638,7 +1759,7 @@ class NDFrameGroupBy(GroupBy):
                 result.insert(0, name, values)
             result.index = np.arange(len(result))
 
-        return result
+        return result.convert_objects()
 
     def _aggregate_multiple_funcs(self, arg):
         from pandas.tools.merge import concat
@@ -1656,9 +1777,10 @@ class NDFrameGroupBy(GroupBy):
                                      grouper=self.grouper)
                 results.append(colg.aggregate(arg))
                 keys.append(col)
-            except (TypeError, DataError):
+            except (TypeError, DataError) :
                 pass
-
+            except SpecificationError:
+                raise
         result = concat(results, keys=keys, axis=1)
 
         return result
@@ -1676,14 +1798,14 @@ class NDFrameGroupBy(GroupBy):
                 for name, data in self:
                     # for name in self.indices:
                     #     data = self.get_group(name, obj=obj)
-                    result[name] = func(data, *args, **kwargs)
+                    result[name] = self._try_cast(func(data, *args, **kwargs),data)
             except Exception:
                 return self._aggregate_item_by_item(func, *args, **kwargs)
         else:
             for name in self.indices:
                 try:
                     data = self.get_group(name, obj=obj)
-                    result[name] = func(data, *args, **kwargs)
+                    result[name] = self._try_cast(func(data, *args, **kwargs), data)
                 except Exception:
                     wrapper = lambda x: func(x, *args, **kwargs)
                     result[name] = data.apply(wrapper, axis=axis)
@@ -1704,7 +1826,7 @@ class NDFrameGroupBy(GroupBy):
                 colg = SeriesGroupBy(obj[item], selection=item,
                                      grouper=self.grouper)
                 result[item] = colg.aggregate(func, *args, **kwargs)
-            except (ValueError, TypeError):
+            except ValueError:
                 cannot_agg.append(item)
                 continue
 
@@ -1760,23 +1882,52 @@ class NDFrameGroupBy(GroupBy):
                     key_index = Index(keys, name=key_names[0])
 
             if isinstance(values[0], np.ndarray):
-                if (isinstance(values[0], Series) and
-                        not _all_indexes_same([x.index for x in values])):
-                    return self._concat_objects(keys, values,
-                                                not_indexed_same=not_indexed_same)
+                if isinstance(values[0], Series):
+                    applied_index    = self.obj._get_axis(self.axis)
+                    all_indexed_same = _all_indexes_same([x.index for x in values])
+                    singular_series  = len(values) == 1 and applied_index.nlevels == 1
 
-                if self.axis == 0:
-                    stacked_values = np.vstack([np.asarray(x)
-                                                for x in values])
-                    columns = values[0].index
-                    index = key_index
-                else:
-                    stacked_values = np.vstack([np.asarray(x)
+                    # GH3596
+                    # provide a reduction (Frame -> Series) if groups are unique
+                    if self.squeeze:
+
+                        # assign the name to this series
+                        if singular_series:
+                            values[0].name = keys[0]
+
+                            # GH2893
+                            # we have series in the values array, we want to produce a series:
+                            # if any of the sub-series are not indexed the same
+                            # OR we don't have a multi-index and we have only a single values
+                            return self._concat_objects(keys, values,
+                                                        not_indexed_same=not_indexed_same)
+
+                    if not all_indexed_same:
+                        return self._concat_objects(keys, values,
+                                                    not_indexed_same=not_indexed_same)
+                    
+                try:
+                    if self.axis == 0:
+
+                        stacked_values = np.vstack([np.asarray(x)
+                                                    for x in values])
+                        columns = values[0].index
+                        index = key_index
+                    else:
+                        stacked_values = np.vstack([np.asarray(x)
                                                 for x in values]).T
-                    index = values[0].index
-                    columns = key_index
+
+                        index = values[0].index
+                        columns = key_index
+
+                except ValueError:
+                    #GH1738,, values is list of arrays of unequal lengths
+                    # fall through to the outer else caluse
+                    return Series(values, index=key_index)
+
                 return DataFrame(stacked_values, index=index,
-                                 columns=columns)
+                                 columns=columns).convert_objects()
+
             else:
                 return Series(values, index=key_index)
         else:
@@ -1800,7 +1951,7 @@ class NDFrameGroupBy(GroupBy):
         Each subframe is endowed the attribute 'name' in case you need to know
         which group you are working on.
 
-        Example
+        Examples
         --------
         >>> grouped = df.groupby(lambda x: mapping[x])
         >>> grouped.transform(lambda x: (x - x.mean()) / x.std())
@@ -1811,21 +1962,23 @@ class NDFrameGroupBy(GroupBy):
 
         obj = self._obj_with_exclusions
         gen = self.grouper.get_iterator(obj, axis=self.axis)
+        fast_path, slow_path = self._define_paths(func, *args, **kwargs)
 
-        if isinstance(func, basestring):
-            wrapper = lambda x: getattr(x, func)(*args, **kwargs)
-        else:
-            wrapper = lambda x: func(x, *args, **kwargs)
-
+        path = None
         for name, group in gen:
             object.__setattr__(group, 'name', name)
 
-            try:
-                res = group.apply(wrapper, axis=self.axis)
-            except TypeError:
-                return self._transform_item_by_item(obj, wrapper)
-            except Exception:  # pragma: no cover
-                res = wrapper(group)
+            if path is None:
+                # Try slow path and fast path.
+                try:
+                    path, res = self._choose_path(fast_path, slow_path, group)
+                except TypeError:
+                    return self._transform_item_by_item(obj, fast_path)
+                except Exception:  # pragma: no cover
+                    res  = fast_path(group)
+                    path = fast_path
+            else:
+                res = path(group)
 
             # broadcasting
             if isinstance(res, Series):
@@ -1841,7 +1994,37 @@ class NDFrameGroupBy(GroupBy):
         concat_index = obj.columns if self.axis == 0 else obj.index
         concatenated = concat(applied, join_axes=[concat_index],
                               axis=self.axis, verify_integrity=False)
-        return concatenated.reindex_like(obj)
+        concatenated.sort_index(inplace=True)
+        return concatenated
+
+    def _define_paths(self, func, *args, **kwargs):
+        if isinstance(func, basestring):
+            fast_path = lambda group: getattr(group, func)(*args, **kwargs)
+            slow_path = lambda group: group.apply(lambda x: getattr(x, func)(*args, **kwargs), axis=self.axis)
+        else:
+            fast_path = lambda group: func(group, *args, **kwargs)
+            slow_path = lambda group: group.apply(lambda x: func(x, *args, **kwargs), axis=self.axis)
+        return fast_path, slow_path 
+
+    def _choose_path(self, fast_path, slow_path, group):
+        path = slow_path
+        res  = slow_path(group)
+
+        # if we make it here, test if we can use the fast path
+        try:
+            res_fast = fast_path(group)
+
+            # compare that we get the same results
+            if res.shape == res_fast.shape:
+                res_r = res.values.ravel()
+                res_fast_r = res_fast.values.ravel()
+                mask = notnull(res_r)
+            if (res_r[mask] == res_fast_r[mask]).all():
+                path = fast_path
+
+        except:
+            pass
+        return path, res
 
     def _transform_item_by_item(self, obj, wrapper):
         # iterate through columns
@@ -1863,6 +2046,63 @@ class NDFrameGroupBy(GroupBy):
 
         return DataFrame(output, index=obj.index, columns=columns)
 
+    def filter(self, func, dropna=True, *args, **kwargs):
+        """
+        Return a copy of a DataFrame excluding elements from groups that
+        do not satisfy the boolean criterion specified by func.
+
+        Parameters
+        ----------
+        f : function
+            Function to apply to each subframe. Should return True or False.
+        dropna : Drop groups that do not pass the filter. True by default;
+            if False, groups that evaluate False are filled with NaNs.
+
+        Note
+        ----
+        Each subframe is endowed the attribute 'name' in case you need to know
+        which group you are working on.
+
+        Example
+        --------
+        >>> grouped = df.groupby(lambda x: mapping[x])
+        >>> grouped.filter(lambda x: x['A'].sum() + x['B'].sum() > 0)
+        """
+        from pandas.tools.merge import concat
+
+        indexers = []
+
+        obj = self._obj_with_exclusions
+        gen = self.grouper.get_iterator(obj, axis=self.axis)
+
+        fast_path, slow_path = self._define_paths(func, *args, **kwargs)
+
+        path = None
+        for name, group in gen:
+            object.__setattr__(group, 'name', name)
+
+            if path is None:
+                # Try slow path and fast path.
+                try:
+                    path, res = self._choose_path(fast_path, slow_path, group)
+                except Exception:  # pragma: no cover
+                    res  = fast_path(group)
+                    path = fast_path
+            else:
+                res = path(group)
+
+            if res:
+                indexers.append(self.obj.index.get_indexer(group.index))
+
+        if len(indexers) == 0:
+            filtered = self.obj.take([]) # because np.concatenate would fail
+        else:
+            filtered = self.obj.take(np.concatenate(indexers))
+        if dropna:
+            return filtered
+        else:
+            return filtered.reindex(self.obj.index) # Fill with NaNs.
+
 
 class DataFrameGroupBy(NDFrameGroupBy):
 
@@ -1928,7 +2168,7 @@ class DataFrameGroupBy(NDFrameGroupBy):
         if self.axis == 1:
             result = result.T
 
-        return result
+        return result.convert_objects()
 
     def _wrap_agged_blocks(self, blocks):
         obj = self._obj_with_exclusions
@@ -1968,7 +2208,7 @@ class DataFrameGroupBy(NDFrameGroupBy):
         if self.axis == 1:
             result = result.T
 
-        return result
+        return result.convert_objects()
 
 
 from pandas.tools.plotting import boxplot_frame_groupby
@@ -2077,7 +2317,7 @@ class DataSplitter(object):
     @cache_readonly
     def slabels(self):
         # Sorted labels
-        return com.ndtake(self.labels, self.sort_idx)
+        return com.take_nd(self.labels, self.sort_idx, allow_fill=False)
 
     @cache_readonly
     def sort_idx(self):
@@ -2355,11 +2595,11 @@ def _reorder_by_uniques(uniques, labels):
     mask = labels < 0
 
     # move labels to right locations (ie, unsort ascending labels)
-    labels = com.ndtake(reverse_indexer, labels)
+    labels = com.take_nd(reverse_indexer, labels, allow_fill=False)
     np.putmask(labels, mask, -1)
 
     # sort observed ids
-    uniques = com.ndtake(uniques, sorter)
+    uniques = com.take_nd(uniques, sorter, allow_fill=False)
 
     return uniques, labels
 
