@@ -7,13 +7,16 @@ import operator
 import sys
 import numpy as np
 from pandas.core.common import (PandasError, _mut_exclusive,
-                                _try_sort, _default_index, _infer_dtype,
+                                _try_sort, _default_index,
+                                _infer_dtype_from_scalar,
                                 notnull)
-from pandas.core.categorical import Factor
+from pandas.core.categorical import Categorical
 from pandas.core.index import (Index, MultiIndex, _ensure_index,
                                _get_combined_index)
-from pandas.core.indexing import _NDFrameIndexer, _maybe_droplevels
-from pandas.core.internals import BlockManager, make_block, form_blocks
+from pandas.core.indexing import _maybe_droplevels, _is_list_like
+from pandas.core.internals import (BlockManager,
+                                   create_block_manager_from_arrays,
+                                   create_block_manager_from_blocks)
 from pandas.core.series import Series
 from pandas.core.frame import DataFrame
 from pandas.core.generic import NDFrame
@@ -79,18 +82,14 @@ def panel_index(time, panels, names=['time', 'panel']):
                 (1962, 'C')], dtype=object)
     """
     time, panels = _ensure_like_indices(time, panels)
-    time_factor = Factor.from_array(time)
-    panel_factor = Factor.from_array(panels)
+    time_factor = Categorical.from_array(time)
+    panel_factor = Categorical.from_array(panels)
 
     labels = [time_factor.labels, panel_factor.labels]
     levels = [time_factor.levels, panel_factor.levels]
     return MultiIndex(levels, labels, sortorder=None, names=names)
 
 
-class PanelError(Exception):
-    pass
-
-
 def _arith_method(func, name):
     # work only for scalars
 
@@ -145,6 +144,24 @@ def _comp_method(func, name):
 
 
 class Panel(NDFrame):
+    """
+    Represents wide format panel data, stored as 3-dimensional array
+
+    Parameters
+    ----------
+    data : ndarray (items x major x minor), or dict of DataFrames
+    items : Index or array-like
+        axis=0
+    major_axis : Index or array-like
+        axis=1
+    minor_axis : Index or array-like
+        axis=2
+    dtype : dtype, default None
+        Data type to force, otherwise infer
+    copy : boolean, default False
+        Copy data from inputs. Only affects DataFrame / 2d ndarray input
+    """
+
     _AXIS_ORDERS = ['items', 'major_axis', 'minor_axis']
     _AXIS_NUMBERS = dict([(a, i) for i, a in enumerate(_AXIS_ORDERS)])
     _AXIS_ALIASES = {
@@ -169,28 +186,24 @@ class Panel(NDFrame):
     major_axis = lib.AxisProperty(1)
     minor_axis = lib.AxisProperty(2)
 
-    @property
-    def _constructor(self):
-        return type(self)
-
     # return the type of the slice constructor
     _constructor_sliced = DataFrame
 
     def _construct_axes_dict(self, axes=None, **kwargs):
-        """ return an axes dictionary for myself """
+        """ Return an axes dictionary for myself """
         d = dict([(a, getattr(self, a)) for a in (axes or self._AXIS_ORDERS)])
         d.update(kwargs)
         return d
 
     @staticmethod
     def _construct_axes_dict_from(self, axes, **kwargs):
-        """ return an axes dictionary for the passed axes """
+        """ Return an axes dictionary for the passed axes """
         d = dict([(a, ax) for a, ax in zip(self._AXIS_ORDERS, axes)])
         d.update(kwargs)
         return d
 
     def _construct_axes_dict_for_slice(self, axes=None, **kwargs):
-        """ return an axes dictionary for myself """
+        """ Return an axes dictionary for myself """
         d = dict([(self._AXIS_SLICEMAP[a], getattr(self, a))
                  for a in (axes or self._AXIS_ORDERS)])
         d.update(kwargs)
@@ -216,29 +229,15 @@ class Panel(NDFrame):
 
     def __init__(self, data=None, items=None, major_axis=None, minor_axis=None,
                  copy=False, dtype=None):
-        """
-        Represents wide format panel data, stored as 3-dimensional array
-
-        Parameters
-        ----------
-        data : ndarray (items x major x minor), or dict of DataFrames
-        items : Index or array-like
-            axis=1
-        major_axis : Index or array-like
-            axis=1
-        minor_axis : Index or array-like
-            axis=2
-        dtype : dtype, default None
-            Data type to force, otherwise infer
-        copy : boolean, default False
-            Copy data from inputs. Only affects DataFrame / 2d ndarray input
-        """
         self._init_data(
             data=data, items=items, major_axis=major_axis, minor_axis=minor_axis,
             copy=copy, dtype=dtype)
 
     def _init_data(self, data, copy, dtype, **kwargs):
-        """ generate ND initialization; axes are passed as required objects to __init__ """
+        """
+        Generate ND initialization; axes are passed
+        as required objects to __init__
+        """
         if data is None:
             data = {}
 
@@ -272,14 +271,19 @@ class Panel(NDFrame):
             return cls(data, **d)
 
     def _init_dict(self, data, axes, dtype=None):
+        from pandas.util.compat import OrderedDict
         haxis = axes.pop(self._het_axis)
 
         # prefilter if haxis passed
         if haxis is not None:
             haxis = _ensure_index(haxis)
-            data = dict((k, v) for k, v in data.iteritems() if k in haxis)
+            data = OrderedDict((k, v) for k, v
+                               in data.iteritems() if k in haxis)
         else:
-            haxis = Index(_try_sort(data.keys()))
+            ks = data.keys()
+            if not isinstance(data,OrderedDict):
+                ks = _try_sort(ks)
+            haxis = Index(ks)
 
         for k, v in data.iteritems():
             if isinstance(v, dict):
@@ -310,14 +314,11 @@ class Panel(NDFrame):
         return self._init_arrays(arrays, haxis, [haxis] + raxes)
 
     def _init_arrays(self, arrays, arr_names, axes):
-        # segregates dtypes and forms blocks matching to columns
-        blocks = form_blocks(arrays, arr_names, axes)
-        mgr = BlockManager(blocks, axes).consolidate()
-        return mgr
+        return create_block_manager_from_arrays(arrays, arr_names, axes)
 
     @property
     def shape(self):
-        return [len(getattr(self, a)) for a in self._AXIS_ORDERS]
+        return tuple([len(getattr(self, a)) for a in self._AXIS_ORDERS])
 
     @classmethod
     def from_dict(cls, data, intersect=False, orient='items', dtype=None):
@@ -342,20 +343,23 @@ class Panel(NDFrame):
         -------
         Panel
         """
-        from collections import defaultdict
+        from pandas.util.compat import OrderedDict,OrderedDefaultdict
 
         orient = orient.lower()
         if orient == 'minor':
-            new_data = defaultdict(dict)
+            new_data = OrderedDefaultdict(dict)
             for col, df in data.iteritems():
                 for item, s in df.iteritems():
                     new_data[item][col] = s
             data = new_data
         elif orient != 'items':  # pragma: no cover
-            raise ValueError('only recognize items or minor for orientation')
+            raise ValueError('Orientation must be one of {items, minor}.')
 
         d = cls._homogenize_dict(cls, data, intersect=intersect, dtype=dtype)
-        d[cls._info_axis] = Index(sorted(d['data'].keys()))
+        ks = d['data'].keys()
+        if not isinstance(d['data'],OrderedDict):
+            ks = list(sorted(ks))
+        d[cls._info_axis] = Index(ks)
         return cls(**d)
 
     def __getitem__(self, key):
@@ -398,9 +402,7 @@ class Panel(NDFrame):
                 ax = _ensure_index(ax)
             fixed_axes.append(ax)
 
-        items = fixed_axes[0]
-        block = make_block(values, items, items)
-        return BlockManager([block], fixed_axes)
+        return create_block_manager_from_blocks([ values ], fixed_axes)
 
     #----------------------------------------------------------------------
     # Array interface
@@ -416,7 +418,8 @@ class Panel(NDFrame):
     # Comparison methods
 
     def _indexed_same(self, other):
-        return all([getattr(self, a).equals(getattr(other, a)) for a in self._AXIS_ORDERS])
+        return all([getattr(self, a).equals(getattr(other, a))
+                    for a in self._AXIS_ORDERS])
 
     def _compare_constructor(self, other, func):
         if not self._indexed_same(other):
@@ -459,33 +462,12 @@ class Panel(NDFrame):
     #----------------------------------------------------------------------
     # Magic methods
 
-    def __str__(self):
-        """
-        Return a string representation for a particular Panel
-
-        Invoked by str(df) in both py2/py3.
-        Yields Bytestring in Py2, Unicode String in py3.
-        """
-
-        if py3compat.PY3:
-            return self.__unicode__()
-        return self.__bytes__()
-
-    def __bytes__(self):
-        """
-        Return a string representation for a particular Panel
-
-        Invoked by bytes(df) in py3 only.
-        Yields a bytestring in both py2/py3.
-        """
-        encoding = com.get_option("display.encoding")
-        return self.__unicode__().encode(encoding, 'replace')
-
     def __unicode__(self):
         """
         Return a string representation for a particular Panel
 
-        Invoked by unicode(df) in py2 only. Yields a Unicode String in both py2/py3.
+        Invoked by unicode(df) in py2 only.
+        Yields a Unicode String in both py2/py3.
         """
 
         class_name = str(self.__class__)
@@ -497,7 +479,9 @@ class Panel(NDFrame):
         def axis_pretty(a):
             v = getattr(self, a)
             if len(v) > 0:
-                return u'%s axis: %s to %s' % (a.capitalize(), com.pprint_thing(v[0]), com.pprint_thing(v[-1]))
+                return u'%s axis: %s to %s' % (a.capitalize(),
+                                               com.pprint_thing(v[0]),
+                                               com.pprint_thing(v[-1]))
             else:
                 return u'%s axis: None' % a.capitalize()
 
@@ -505,14 +489,6 @@ class Panel(NDFrame):
             [class_name, dims] + [axis_pretty(a) for a in self._AXIS_ORDERS])
         return output
 
-    def __repr__(self):
-        """
-        Return a string representation for a particular Panel
-
-        Yields Bytestring in Py2, Unicode String in py3.
-        """
-        return str(self)
-
     def __iter__(self):
         return iter(getattr(self, self._info_axis))
 
@@ -525,7 +501,11 @@ class Panel(NDFrame):
     iterkv = iteritems
 
     def _get_plane_axes(self, axis):
-        """ get my plane axes: these are already (as compared with higher level planes), as we are returning a DataFrame axes """
+        """
+        Get my plane axes: these are already
+        (as compared with higher level planes),
+        as we are returning a DataFrame axes
+        """
         axis = self._get_axis_name(axis)
 
         if axis == 'major_axis':
@@ -540,16 +520,6 @@ class Panel(NDFrame):
 
         return index, columns
 
-    # Fancy indexing
-    _ix = None
-
-    @property
-    def ix(self):
-        if self._ix is None:
-            self._ix = _NDFrameIndexer(self)
-
-        return self._ix
-
     def _wrap_array(self, arr, axes, copy=False):
         d = self._construct_axes_dict_from(self, axes, copy=copy)
         return self._constructor(arr, **d)
@@ -588,7 +558,7 @@ class Panel(NDFrame):
         na_rep : string, default ''
             Missing data representation
         """
-        from pandas.io.parsers import ExcelWriter
+        from pandas.io.excel import ExcelWriter
         writer = ExcelWriter(path)
         for item, df in self.iteritems():
             name = str(item)
@@ -623,7 +593,8 @@ class Panel(NDFrame):
         value : scalar value
         """
         # require an arg for each axis
-        assert(len(args) == self._AXIS_LEN)
+        if not ((len(args) == self._AXIS_LEN)):
+            raise AssertionError()
 
         # hm, two layers to the onion
         frame = self._get_item_cache(args[0])
@@ -647,7 +618,8 @@ class Panel(NDFrame):
             otherwise a new object
         """
         # require an arg for each axis and the value
-        assert(len(args) == self._AXIS_LEN + 1)
+        if not ((len(args) == self._AXIS_LEN + 1)):
+            raise AssertionError()
 
         try:
             frame = self._get_item_cache(args[0])
@@ -657,8 +629,8 @@ class Panel(NDFrame):
             axes = self._expand_axes(args)
             d = self._construct_axes_dict_from(self, axes, copy=False)
             result = self.reindex(**d)
-
-            likely_dtype = com._infer_dtype(args[-1])
+            args  = list(args)
+            likely_dtype, args[-1] = _infer_dtype_from_scalar(args[-1])
             made_bigger = not np.array_equal(
                 axes[0], getattr(self, self._info_axis))
             # how to make this logic simpler?
@@ -679,8 +651,10 @@ class Panel(NDFrame):
         raise AttributeError("'%s' object has no attribute '%s'" %
                              (type(self).__name__, name))
 
-    def _slice(self, slobj, axis=0):
-        new_data = self._data.get_slice(slobj, axis=axis)
+    def _slice(self, slobj, axis=0, raise_on_error=False):
+        new_data = self._data.get_slice(slobj,
+                                        axis=axis,
+                                        raise_on_error=raise_on_error)
         return self._constructor(new_data)
 
     def __setitem__(self, key, value):
@@ -690,10 +664,11 @@ class Panel(NDFrame):
                 **self._construct_axes_dict_for_slice(self._AXIS_ORDERS[1:]))
             mat = value.values
         elif isinstance(value, np.ndarray):
-            assert(value.shape == shape[1:])
+            if not ((value.shape == shape[1:])):
+                raise AssertionError()
             mat = np.asarray(value)
         elif np.isscalar(value):
-            dtype = _infer_dtype(value)
+            dtype, value = _infer_dtype_from_scalar(value)
             mat = np.empty(shape[1:], dtype=dtype)
             mat.fill(value)
         else:
@@ -838,7 +813,7 @@ class Panel(NDFrame):
             indexer2 = range(len(new_minor))
 
         for i, ind in enumerate(indexer0):
-            com.take_2d_multi(values[ind], indexer1, indexer2,
+            com.take_2d_multi(values[ind], (indexer1, indexer2),
                               out=new_values[i])
 
         return Panel(new_values, items=new_items, major_axis=new_major,
@@ -994,6 +969,9 @@ class Panel(NDFrame):
         --------
         DataFrame.reindex, DataFrame.asfreq
         """
+        if isinstance(value, (list, tuple)):
+            raise TypeError('"value" parameter must be a scalar or dict, but '
+                            'you passed a "{0}"'.format(type(value).__name__))
         if value is None:
             if method is None:
                 raise ValueError('must specify a fill method or value')
@@ -1064,6 +1042,7 @@ class Panel(NDFrame):
         -------
         y : ndim(self)-1
         """
+        axis = self._get_axis_number(axis)
         if axis == 0:
             data = self[key]
             if copy:
@@ -1075,10 +1054,17 @@ class Panel(NDFrame):
         new_data = self._data.xs(key, axis=axis_number, copy=copy)
         return self._constructor_sliced(new_data)
 
+    _xs = xs
+
     def _ixs(self, i, axis=0):
         # for compatibility with .ix indexing
         # Won't work with hierarchical indexing yet
         key = self._get_axis(axis)[i]
+
+        # xs cannot handle a non-scalar key, so just reindex here
+        if _is_list_like(key):
+            return self.reindex(**{ self._get_axis_name(axis) : key })
+
         return self.xs(key, axis=axis)
 
     def groupby(self, function, axis='major'):
@@ -1145,16 +1131,35 @@ class Panel(NDFrame):
         -------
         y : Panel (new object)
         """
-
         # construct the args
         args = list(args)
+        aliases = tuple(kwargs.iterkeys())
+
         for a in self._AXIS_ORDERS:
             if not a in kwargs:
-                try:
-                    kwargs[a] = args.pop(0)
-                except (IndexError):
-                    raise ValueError(
-                        "not enough arguments specified to transpose!")
+                where = map(a.startswith, aliases)
+
+                if any(where):
+                    if sum(where) != 1:
+                        raise AssertionError(
+                            'Ambiguous parameter aliases "{0}" passed, valid '
+                            'parameter aliases are '
+                            '{1}'.format([n for n, m in zip(aliases, where)
+                                          if m], self._AXIS_ALIASES))
+
+                    k = aliases[where.index(True)]
+
+                    try:
+                        kwargs[self._AXIS_ALIASES[k]] = kwargs.pop(k)
+                    except KeyError:
+                        raise KeyError('Invalid parameter alias '
+                                       '"{0}"'.format(k))
+                else:
+                    try:
+                        kwargs[a] = args.pop(0)
+                    except IndexError:
+                        raise ValueError(
+                            "not enough arguments specified to transpose!")
 
         axes = [self._get_axis_number(kwargs[a]) for a in self._AXIS_ORDERS]
 
@@ -1262,7 +1267,8 @@ class Panel(NDFrame):
         if result.ndim == 2 and axis_name != self._info_axis:
             result = result.T
 
-        return self._constructor_sliced(result, **self._extract_axes_for_slice(self, axes))
+        return self._constructor_sliced(result,
+                                **self._extract_axes_for_slice(self, axes))
 
     def _wrap_result(self, result, axis):
         axis = self._get_axis_name(axis)
@@ -1274,7 +1280,9 @@ class Panel(NDFrame):
         if self.ndim == result.ndim:
             return self._constructor(result, **self._construct_axes_dict())
         elif self.ndim == result.ndim + 1:
-            return self._constructor_sliced(result, **self._extract_axes_for_slice(self, axes))
+            return self._constructor_sliced(result,
+                                **self._extract_axes_for_slice(self, axes))
+
         raise PandasError("invalid _wrap_result [self->%s] [result->%s]" %
                           (self.ndim, result.ndim))
 
@@ -1326,10 +1334,11 @@ class Panel(NDFrame):
             vslicer = slice(-lags, None)
             islicer = slice(None, lags)
 
-        if axis == 'major':
+        axis = self._get_axis_name(axis)
+        if axis == 'major_axis':
             values = values[:, vslicer, :]
             major_axis = major_axis[islicer]
-        elif axis == 'minor':
+        elif axis == 'minor_axis':
             values = values[:, :, vslicer]
             minor_axis = minor_axis[islicer]
         else:
@@ -1460,12 +1469,14 @@ class Panel(NDFrame):
     @staticmethod
     def _extract_axes(self, data, axes, **kwargs):
         """ return a list of the axis indicies """
-        return [self._extract_axis(self, data, axis=i, **kwargs) for i, a in enumerate(axes)]
+        return [self._extract_axis(self, data, axis=i, **kwargs) for i, a
+                in enumerate(axes)]
 
     @staticmethod
     def _extract_axes_for_slice(self, axes):
         """ return the slice dictionary for these axes """
-        return dict([(self._AXIS_SLICEMAP[i], a) for i, a in zip(self._AXIS_ORDERS[self._AXIS_LEN - len(axes):], axes)])
+        return dict([(self._AXIS_SLICEMAP[i], a) for i, a
+                     in zip(self._AXIS_ORDERS[self._AXIS_LEN - len(axes):], axes)])
 
     @staticmethod
     def _prep_ndarray(self, values, copy=True):
@@ -1477,14 +1488,15 @@ class Panel(NDFrame):
         else:
             if copy:
                 values = values.copy()
-        assert(values.ndim == self._AXIS_LEN)
+        if not ((values.ndim == self._AXIS_LEN)):
+            raise AssertionError()
         return values
 
     @staticmethod
     def _homogenize_dict(self, frames, intersect=True, dtype=None):
         """
-        Conform set of _constructor_sliced-like objects to either an intersection
-        of indices / columns or a union.
+        Conform set of _constructor_sliced-like objects to either
+        an intersection of indices / columns or a union.
 
         Parameters
         ----------
@@ -1495,9 +1507,13 @@ class Panel(NDFrame):
         -------
         dict of aligned results & indicies
         """
-        result = {}
+        from pandas.util.compat import OrderedDict
 
-        adj_frames = {}
+        result = dict()
+        if isinstance(frames,OrderedDict): # caller differs dict/ODict, presered type
+            result = OrderedDict()
+
+        adj_frames = OrderedDict()
         for k, v in frames.iteritems():
             if isinstance(v, dict):
                 adj_frames[k] = self._constructor_sliced(v)
@@ -1602,7 +1618,8 @@ Return %(desc)s over requested axis
 
 Parameters
 ----------
-axis : {""" + ', '.join(cls._AXIS_ORDERS) + "} or {" + ', '.join([str(i) for i in range(cls._AXIS_LEN)]) + """}
+axis : {""" + ', '.join(cls._AXIS_ORDERS) + "} or {" \
++ ', '.join([str(i) for i in range(cls._AXIS_LEN)]) + """}
 skipna : boolean, default True
     Exclude NA/null values. If an entire row/column is NA, the result
     will be NA
@@ -1694,7 +1711,8 @@ def install_ipython_completers():  # pragma: no cover
     @complete_object.when_type(Panel)
     def complete_dataframe(obj, prev_completions):
         return prev_completions + [c for c in obj.keys()
-                                   if isinstance(c, basestring) and py3compat.isidentifier(c)]
+                                   if isinstance(c, basestring)
+                                        and py3compat.isidentifier(c)]
 
 # Importing IPython brings in about 200 modules, so we want to avoid it unless
 # we're in IPython (when those modules are loaded anyway).
