@@ -137,7 +137,8 @@ def rolling_count(arg, window, freq=None, center=False, time_rule=None):
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     rolling_count : type of caller
@@ -166,18 +167,11 @@ def rolling_cov(arg1, arg2, window, min_periods=None, freq=None,
     window = min(window, len(arg1), len(arg2))
 
     def _get_cov(X, Y):
-        mean = lambda x: rolling_mean(x, window, min_periods)
-        count = rolling_count(X + Y, window)
+        mean = lambda x: rolling_mean(x, window, min_periods,center=center)
+        count = rolling_count(X + Y, window,center=center)
         bias_adj = count / (count - 1)
         return (mean(X * Y) - mean(X) * mean(Y)) * bias_adj
     rs = _flex_binary_moment(arg1, arg2, _get_cov)
-    if center:
-        if isinstance(rs, (Series, DataFrame, Panel)):
-            rs = rs.shift(-int((window + 1) / 2.))
-        else:
-            offset = int((window + 1) / 2.)
-            rs[:-offset] = rs[offset:]
-            rs[-offset:] = np.nan
     return rs
 
 
@@ -197,6 +191,10 @@ def rolling_corr(arg1, arg2, window, min_periods=None, freq=None,
 
 
 def _flex_binary_moment(arg1, arg2, f):
+    if not (isinstance(arg1,(np.ndarray, DataFrame)) and
+            isinstance(arg1,(np.ndarray, DataFrame))):
+        raise ValueError("arguments to moment function must be of type ndarray/DataFrame")
+
     if isinstance(arg1, np.ndarray) and isinstance(arg2, np.ndarray):
         X, Y = _prep_binary(arg1, arg2)
         return f(X, Y)
@@ -271,7 +269,8 @@ def _rolling_moment(arg, window, func, minp, axis=0, freq=None,
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     y : type of input
@@ -292,6 +291,9 @@ def _rolling_moment(arg, window, func, minp, axis=0, freq=None,
 
 
 def _center_window(rs, window, axis):
+    if axis > rs.ndim-1:
+        raise ValueError("Requested axis is larger then no. of argument dimensions")
+
     offset = int((window - 1) / 2.)
     if isinstance(rs, (Series, DataFrame, Panel)):
         rs = rs.shift(-offset, axis=axis)
@@ -540,7 +542,8 @@ def rolling_quantile(arg, window, quantile, min_periods=None, freq=None,
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     y : type of input argument
@@ -569,7 +572,8 @@ def rolling_apply(arg, window, func, min_periods=None, freq=None,
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     y : type of input argument
@@ -604,7 +608,9 @@ def rolling_window(arg, window=None, win_type=None, min_periods=None,
         Whether the label should correspond with center of window
     mean : boolean, default True
         If True computes weighted mean, else weighted sum
-
+    time_rule : Legacy alias for freq
+    axis : {0, 1}, default 0
+    
     Returns
     -------
     y : type of input argument
@@ -729,7 +735,8 @@ def expanding_count(arg, freq=None, center=False, time_rule=None):
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     expanding_count : type of caller
@@ -752,7 +759,8 @@ def expanding_quantile(arg, quantile, min_periods=1, freq=None,
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     y : type of input argument
@@ -816,7 +824,8 @@ def expanding_apply(arg, func, min_periods=1, freq=None, center=False,
         Frequency to conform to before computing statistic
     center : boolean, default False
         Whether the label should correspond with center of window
-
+    time_rule : Legacy alias for freq
+    
     Returns
     -------
     y : type of input argument
