@@ -18,7 +18,11 @@ from pandas.util import py3compat
 from pandas.util.testing import (assert_panel_equal,
                                  assert_frame_equal,
                                  assert_series_equal,
-                                 assert_almost_equal)
+                                 assert_almost_equal,
+                                 ensure_clean,
+                                 assertRaisesRegexp,
+                                 makeCustomDataframe as mkdf
+    )
 import pandas.core.panel as panelm
 import pandas.util.testing as tm
 
@@ -43,6 +47,12 @@ class PanelTests(object):
         cumsum = self.panel.cumsum()
         assert_frame_equal(cumsum['ItemA'], self.panel['ItemA'].cumsum())
 
+    def not_hashable(self):
+        c_empty = Panel()
+        c = Panel(pd.Panel([[[1]]]))
+        self.assertRaises(TypeError, hash, c_empty)
+        self.assertRaises(TypeError, hash, c)
+
 
 class SafeForLongAndSparse(object):
     _multiprocess_can_split_ = True
@@ -782,6 +792,13 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         assert_almost_equal(casted.values, exp_values)
         assert_almost_equal(casted2.values, exp_values)
 
+        casted = Panel(zero_filled._data, dtype=np.int32)
+        casted2 = Panel(zero_filled.values, dtype=np.int32)
+
+        exp_values = zero_filled.values.astype(np.int32)
+        assert_almost_equal(casted.values, exp_values)
+        assert_almost_equal(casted2.values, exp_values)
+
         # can't cast
         data = [[['foo', 'bar', 'baz']]]
         self.assertRaises(ValueError, Panel, data, dtype=float)
@@ -798,6 +815,30 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
                       minor_axis=range(3), dtype='O')
         self.assert_(panel.values.dtype == np.object_)
 
+    def test_constructor_dtypes(self):
+        # GH #797
+
+        def _check_dtype(panel, dtype):
+            for i in panel.items:
+                self.assert_(panel[i].values.dtype.name == dtype)
+
+        # only nan holding types allowed here
+        for dtype in ['float64','float32','object']:
+            panel = Panel(items=range(2),major_axis=range(10),minor_axis=range(5),dtype=dtype)
+            _check_dtype(panel,dtype)
+
+        for dtype in ['float64','float32','int64','int32','object']:
+            panel = Panel(np.array(np.random.randn(2,10,5),dtype=dtype),items=range(2),major_axis=range(10),minor_axis=range(5),dtype=dtype)
+            _check_dtype(panel,dtype)
+
+        for dtype in ['float64','float32','int64','int32','object']:
+            panel = Panel(np.array(np.random.randn(2,10,5),dtype='O'),items=range(2),major_axis=range(10),minor_axis=range(5),dtype=dtype)
+            _check_dtype(panel,dtype)
+
+        for dtype in ['float64','float32','int64','int32','object']:
+            panel = Panel(np.random.randn(2,10,5),items=range(2),major_axis=range(10),minor_axis=range(5),dtype=dtype)
+            _check_dtype(panel,dtype)
+
     def test_consolidate(self):
         self.assert_(self.panel._data.is_consolidated())
 
@@ -845,6 +886,11 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
                               for k, v in dcasted.iteritems()))
         assert_panel_equal(result, expected)
 
+        result = Panel(dcasted, dtype=np.int32)
+        expected = Panel(dict((k, v.astype(np.int32))
+                              for k, v in dcasted.iteritems()))
+        assert_panel_equal(result, expected)
+
     def test_constructor_dict_mixed(self):
         data = dict((k, v.values) for k, v in self.panel.iterkv())
         result = Panel(data)
@@ -867,6 +913,16 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         data['ItemB'] = self.panel['ItemB'].values[:, :-1]
         self.assertRaises(Exception, Panel, data)
 
+    def test_ctor_orderedDict(self):
+        from pandas.util.compat import OrderedDict
+        keys = list(set(np.random.randint(0,5000,100)))[:50] # unique random int  keys
+        d = OrderedDict([(k,mkdf(10,5)) for k in keys])
+        p = Panel(d)
+        self.assertTrue(list(p.items) == keys)
+
+        p = Panel.from_dict(d)
+        self.assertTrue(list(p.items) == keys)
+
     def test_constructor_resize(self):
         data = self.panel._data
         items = self.panel.items[:-1]
@@ -902,9 +958,19 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         self.assert_(panel['foo'].values.dtype == np.object_)
         self.assert_(panel['A'].values.dtype == np.float64)
 
-    def test_values(self):
-        self.assertRaises(Exception, Panel, np.random.randn(5, 5, 5),
-                          range(5), range(5), range(4))
+    def test_constructor_error_msgs(self):
+
+        def testit():
+            Panel(np.random.randn(3,4,5), range(4), range(5), range(5))
+        assertRaisesRegexp(ValueError, "Shape of passed values is \(3, 4, 5\), indices imply \(4, 5, 5\)", testit)
+
+        def testit():
+            Panel(np.random.randn(3,4,5), range(5), range(4), range(5))
+        assertRaisesRegexp(ValueError, "Shape of passed values is \(3, 4, 5\), indices imply \(5, 4, 5\)", testit)
+
+        def testit():
+            Panel(np.random.randn(3,4,5), range(5), range(5), range(4))
+        assertRaisesRegexp(ValueError, "Shape of passed values is \(3, 4, 5\), indices imply \(5, 5, 4\)", testit)
 
     def test_conform(self):
         df = self.panel['ItemA'][:-5].filter(items=['A', 'B'])
@@ -977,7 +1043,11 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         expected = self.panel.reindex(minor=['D', 'A', 'B', 'C'])
         assert_panel_equal(result, expected)
 
-        self.assertRaises(Exception, self.panel.take, [3, -1, 1, 2], axis=2)
+        # neg indicies ok
+        expected = self.panel.reindex(minor=['D', 'D', 'B', 'C'])
+        result = self.panel.take([3, -1, 1, 2], axis=2)
+        assert_panel_equal(result, expected)
+
         self.assertRaises(Exception, self.panel.take, [4, 0, 1, 2], axis=2)
 
     def test_sort_index(self):
@@ -1030,6 +1100,9 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         self.assertRaises(ValueError, self.panel.fillna)
         self.assertRaises(ValueError, self.panel.fillna, 5, method='ffill')
 
+        self.assertRaises(TypeError, self.panel.fillna, [1, 2])
+        self.assertRaises(TypeError, self.panel.fillna, (1, 2))
+
     def test_ffill_bfill(self):
         assert_panel_equal(self.panel.ffill(),
                            self.panel.fillna(method='ffill'))
@@ -1071,6 +1144,30 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
         expected = self.panel.swapaxes('items', 'minor')
         assert_panel_equal(result, expected)
 
+        # test kwargs
+        result = self.panel.transpose(items='minor', major='major',
+                                      minor='items')
+        expected = self.panel.swapaxes('items', 'minor')
+        assert_panel_equal(result, expected)
+
+        # text mixture of args
+        result = self.panel.transpose('minor', major='major', minor='items')
+        expected = self.panel.swapaxes('items', 'minor')
+        assert_panel_equal(result, expected)
+
+        result = self.panel.transpose('minor', 'major', minor='items')
+        expected = self.panel.swapaxes('items', 'minor')
+        assert_panel_equal(result, expected)
+
+        ## test bad aliases
+        # test ambiguous aliases
+        self.assertRaises(AssertionError, self.panel.transpose, 'minor',
+                          maj='major', majo='items')
+
+        # test invalid kwargs
+        self.assertRaises(KeyError, self.panel.transpose, 'minor',
+                          maj='major', minor='items')
+
         result = self.panel.transpose(2, 1, 0)
         assert_panel_equal(result, expected)
 
@@ -1271,18 +1368,22 @@ class TestPanel(unittest.TestCase, PanelTests, CheckIndexing,
             import xlwt
             import xlrd
             import openpyxl
-            from pandas.io.parsers import ExcelFile
+            from pandas.io.excel import ExcelFile
         except ImportError:
             raise nose.SkipTest
 
         for ext in ['xls', 'xlsx']:
             path = '__tmp__.' + ext
-            self.panel.to_excel(path)
-            reader = ExcelFile(path)
-            for item, df in self.panel.iterkv():
-                recdf = reader.parse(str(item), index_col=0)
-                assert_frame_equal(df, recdf)
-            os.remove(path)
+            with ensure_clean(path) as path:
+                self.panel.to_excel(path)
+                try:
+                    reader = ExcelFile(path)
+                except ImportError:
+                    raise nose.SkipTest
+
+                for item, df in self.panel.iterkv():
+                    recdf = reader.parse(str(item), index_col=0)
+                    assert_frame_equal(df, recdf)
 
     def test_dropna(self):
         p = Panel(np.random.randn(4, 5, 6), major_axis=list('abcde'))
@@ -1526,17 +1627,17 @@ class TestLongPanel(unittest.TestCase):
         trunced = self.panel.truncate(start, end).to_panel()
         expected = self.panel.to_panel()['ItemA'].truncate(start, end)
 
-        assert_frame_equal(trunced['ItemA'], expected)
+        assert_frame_equal(trunced['ItemA'], expected, check_names=False)  # TODO trucate drops index.names
 
         trunced = self.panel.truncate(before=start).to_panel()
         expected = self.panel.to_panel()['ItemA'].truncate(before=start)
 
-        assert_frame_equal(trunced['ItemA'], expected)
+        assert_frame_equal(trunced['ItemA'], expected, check_names=False)  # TODO trucate drops index.names
 
         trunced = self.panel.truncate(after=end).to_panel()
         expected = self.panel.to_panel()['ItemA'].truncate(after=end)
 
-        assert_frame_equal(trunced['ItemA'], expected)
+        assert_frame_equal(trunced['ItemA'], expected, check_names=False)  # TODO trucate drops index.names
 
         # truncate on dates that aren't in there
         wp = self.panel.to_panel()
