@@ -15,9 +15,17 @@ from pandas.tseries.offsets import DateOffset
 from pandas.tseries.period import period_range, Period, PeriodIndex
 from pandas.tseries.resample import DatetimeIndex
 
-from pandas.util.testing import assert_series_equal
+from pandas.util.testing import assert_series_equal, ensure_clean
 import pandas.util.testing as tm
 
+
+def _skip_if_no_scipy():
+    try:
+        import scipy
+    except ImportError:
+        raise nose.SkipTest
+
+
 class TestTSPlot(unittest.TestCase):
 
     @classmethod
@@ -34,14 +42,21 @@ class TestTSPlot(unittest.TestCase):
         self.period_ser = [Series(np.random.randn(len(x)), x) for x in idx]
         self.period_df = [DataFrame(np.random.randn(len(x), 3), index=x,
                                     columns=['A', 'B', 'C'])
-                                    for x in idx]
+                          for x in idx]
 
         freq = ['S', 'T', 'H', 'D', 'W', 'M', 'Q-DEC', 'A', '1B30Min']
         idx = [date_range('12/31/1999', freq=x, periods=100) for x in freq]
         self.datetime_ser = [Series(np.random.randn(len(x)), x) for x in idx]
         self.datetime_df = [DataFrame(np.random.randn(len(x), 3), index=x,
-                                    columns=['A', 'B', 'C'])
-                                    for x in idx]
+                                      columns=['A', 'B', 'C'])
+                            for x in idx]
+
+    @slow
+    def test_ts_plot_with_tz(self):
+        # GH2877
+        index = date_range('1/1/2011', periods=2, freq='H', tz='Europe/Brussels')
+        ts = Series([188.5, 328.25], index=index)
+        ts.plot()
 
     @slow
     def test_frame_inferred(self):
@@ -65,6 +80,22 @@ class TestTSPlot(unittest.TestCase):
         df = DataFrame(np.random.randn(len(idx), 3), index=idx)
         df.plot()
 
+    @slow
+    def test_nonnumeric_exclude(self):
+        import matplotlib.pyplot as plt
+        plt.close('all')
+
+        idx = date_range('1/1/1987', freq='A', periods=3)
+        df = DataFrame({'A': ["x", "y", "z"], 'B': [1,2,3]}, idx)
+
+        plt.close('all')
+        ax = df.plot() # it works
+        self.assert_(len(ax.get_lines()) == 1) #B was plotted
+
+        plt.close('all')
+
+        self.assertRaises(TypeError, df['A'].plot)
+
     @slow
     def test_tsplot(self):
         from pandas.tseries.plotting import tsplot
@@ -89,6 +120,18 @@ class TestTSPlot(unittest.TestCase):
         ax = ts.plot(style='k')
         self.assert_((0., 0., 0.) == ax.get_lines()[0].get_color())
 
+    @slow
+    def test_both_style_and_color(self):
+        import matplotlib.pyplot as plt
+        plt.close('all')
+
+        ts = tm.makeTimeSeries()
+        ts.plot(style='b-', color='#000099')  # works
+
+        plt.close('all')
+        s = ts.reset_index(drop=True)
+        s.plot(style='b-', color='#000099')  # non-tsplot
+
     @slow
     def test_high_freq(self):
         freaks = ['ms', 'us']
@@ -202,7 +245,7 @@ class TestTSPlot(unittest.TestCase):
         diffs = Series(ax.get_lines()[0].get_xydata()[:, 0]).diff()
 
         sec = 1. / 24 / 60 / 60
-        self.assert_((np.fabs(diffs[1:] - [sec, sec*2, sec]) < 1e-8).all())
+        self.assert_((np.fabs(diffs[1:] - [sec, sec * 2, sec]) < 1e-8).all())
 
         plt.clf()
         fig.add_subplot(111)
@@ -216,7 +259,7 @@ class TestTSPlot(unittest.TestCase):
     def test_irregular_datetime64_repr_bug(self):
         import matplotlib.pyplot as plt
         ser = tm.makeTimeSeries()
-        ser = ser[[0,1,2,7]]
+        ser = ser[[0, 1, 2, 7]]
 
         fig = plt.gcf()
         plt.clf()
@@ -252,6 +295,18 @@ class TestTSPlot(unittest.TestCase):
         idx = ax.get_lines()[0].get_xdata()
         self.assert_(PeriodIndex(data=idx).freqstr == 'M')
 
+    @slow
+    def test_nonzero_base(self):
+        import matplotlib.pyplot as plt
+        plt.close('all')
+        #GH2571
+        idx = (date_range('2012-12-20', periods=24, freq='H') +
+               timedelta(minutes=30))
+        df = DataFrame(np.arange(24), index=idx)
+        ax = df.plot()
+        rs = ax.get_lines()[0].get_xdata()
+        self.assert_(not Index(rs).is_normalized)
+
     @slow
     def test_dataframe(self):
         bts = DataFrame({'a': tm.makeTimeSeries()})
@@ -293,11 +348,11 @@ class TestTSPlot(unittest.TestCase):
         ax = ser.plot()
         _test(ax)
 
-        df = DataFrame({'a' : ser, 'b' : ser + 1})
+        df = DataFrame({'a': ser, 'b': ser + 1})
         ax = df.plot()
         _test(ax)
 
-        df = DataFrame({'a' : ser, 'b' : ser + 1})
+        df = DataFrame({'a': ser, 'b': ser + 1})
         axes = df.plot(subplots=True)
         [_test(ax) for ax in axes]
 
@@ -368,7 +423,7 @@ class TestTSPlot(unittest.TestCase):
     def test_finder_monthly_long(self):
         import matplotlib.pyplot as plt
         plt.close('all')
-        rng = period_range('1988Q1', periods=24*12, freq='M')
+        rng = period_range('1988Q1', periods=24 * 12, freq='M')
         ser = Series(np.random.randn(len(rng)), rng)
         ax = ser.plot()
         xaxis = ax.get_xaxis()
@@ -530,6 +585,8 @@ class TestTSPlot(unittest.TestCase):
 
     @slow
     def test_secondary_kde(self):
+        _skip_if_no_scipy()
+
         import matplotlib.pyplot as plt
         plt.close('all')
         ser = Series(np.random.randn(10))
@@ -558,6 +615,16 @@ class TestTSPlot(unittest.TestCase):
         self.assert_(axes[1].get_yaxis().get_ticks_position() == 'default')
         self.assert_(axes[2].get_yaxis().get_ticks_position() == 'right')
 
+    @slow
+    def test_secondary_bar_frame(self):
+        import matplotlib.pyplot as plt
+        plt.close('all')
+        df = DataFrame(np.random.randn(5, 3), columns=['a', 'b', 'c'])
+        axes = df.plot(kind='bar', secondary_y=['a', 'c'], subplots=True)
+        self.assert_(axes[0].get_yaxis().get_ticks_position() == 'right')
+        self.assert_(axes[1].get_yaxis().get_ticks_position() == 'default')
+        self.assert_(axes[2].get_yaxis().get_ticks_position() == 'right')
+
     @slow
     def test_mixed_freq_regular_first(self):
         import matplotlib.pyplot as plt
@@ -606,6 +673,21 @@ class TestTSPlot(unittest.TestCase):
         for l in ax.get_lines():
             self.assert_(PeriodIndex(data=l.get_xdata()).freq == 'D')
 
+    @slow
+    def test_mixed_freq_alignment(self):
+        import matplotlib.pyplot as plt
+        ts_ind = date_range('2012-01-01 13:00', '2012-01-02', freq='H')
+        ts_data = np.random.randn(12)
+
+        ts = Series(ts_data, index=ts_ind)
+        ts2 = ts.asfreq('T').interpolate()
+
+        plt.close('all')
+        ax = ts.plot()
+        ts2.plot(style='r')
+
+        self.assert_(ax.lines[0].get_xdata()[0] == ax.lines[1].get_xdata()[0])
+
     @slow
     def test_mixed_freq_lf_first(self):
         import matplotlib.pyplot as plt
@@ -668,12 +750,12 @@ class TestTSPlot(unittest.TestCase):
 
     @slow
     def test_irreg_dtypes(self):
-        #date
+        # date
         idx = [date(2000, 1, 1), date(2000, 1, 5), date(2000, 1, 20)]
         df = DataFrame(np.random.randn(len(idx), 3), Index(idx, dtype=object))
         _check_plot_works(df.plot)
 
-        #np.datetime64
+        # np.datetime64
         idx = date_range('1/1/2000', periods=10)
         idx = idx[[0, 2, 5, 9]].asobject
         df = DataFrame(np.random.randn(len(idx), 3), idx)
@@ -687,8 +769,8 @@ class TestTSPlot(unittest.TestCase):
         t = datetime(1, 1, 1, 3, 30, 0)
         deltas = np.random.randint(1, 20, 3).cumsum()
         ts = np.array([(t + timedelta(minutes=int(x))).time() for x in deltas])
-        df = DataFrame({'a' : np.random.randn(len(ts)),
-                        'b' : np.random.randn(len(ts))},
+        df = DataFrame({'a': np.random.randn(len(ts)),
+                        'b': np.random.randn(len(ts))},
                        index=ts)
         ax = df.plot()
 
@@ -726,8 +808,8 @@ class TestTSPlot(unittest.TestCase):
         deltas = np.random.randint(1, 20, 3).cumsum()
         ts = np.array([(t + timedelta(microseconds=int(x))).time()
                        for x in deltas])
-        df = DataFrame({'a' : np.random.randn(len(ts)),
-                        'b' : np.random.randn(len(ts))},
+        df = DataFrame({'a': np.random.randn(len(ts)),
+                        'b': np.random.randn(len(ts))},
                        index=ts)
         ax = df.plot()
 
@@ -765,7 +847,7 @@ class TestTSPlot(unittest.TestCase):
         plt.clf()
         ax = fig.add_subplot(211)
 
-        #ts
+        # ts
         df = tm.makeTimeDataFrame()
         ax = df.plot(secondary_y=['A', 'B'])
         leg = ax.get_legend()
@@ -792,6 +874,18 @@ class TestTSPlot(unittest.TestCase):
         self.assert_(leg.get_texts()[2].get_text() == 'C')
         self.assert_(leg.get_texts()[3].get_text() == 'D')
 
+        plt.clf()
+        ax = df.plot(kind='bar', secondary_y=['A'])
+        leg = ax.get_legend()
+        self.assert_(leg.get_texts()[0].get_text() == 'A (right)')
+        self.assert_(leg.get_texts()[1].get_text() == 'B')
+
+        plt.clf()
+        ax = df.plot(kind='bar', secondary_y=['A'], mark_right=False)
+        leg = ax.get_legend()
+        self.assert_(leg.get_texts()[0].get_text() == 'A')
+        self.assert_(leg.get_texts()[1].get_text() == 'B')
+
         plt.clf()
         ax = fig.add_subplot(211)
         df = tm.makeTimeDataFrame()
@@ -806,7 +900,7 @@ class TestTSPlot(unittest.TestCase):
         # TODO: color cycle problems
         self.assert_(len(colors) == 4)
 
-        #non-ts
+        # non-ts
         df = tm.makeDataFrame()
         plt.clf()
         ax = fig.add_subplot(211)
@@ -877,7 +971,6 @@ class TestTSPlot(unittest.TestCase):
         assert_array_equal(np.array([x.toordinal() for x in dates]),
                            line2.get_xydata()[:, 0])
 
-PNG_PATH = 'tmp.png'
 def _check_plot_works(f, freq=None, series=None, *args, **kwargs):
     import matplotlib.pyplot as plt
 
@@ -908,9 +1001,10 @@ def _check_plot_works(f, freq=None, series=None, *args, **kwargs):
         assert(ret is not None)  # do something more intelligent
     except Exception:
         pass
-    plt.savefig(PNG_PATH)
-    os.remove(PNG_PATH)
+
+    with ensure_clean() as path:
+        plt.savefig(path)
 
 if __name__ == '__main__':
-    nose.runmodule(argv=[__file__,'-vvs','-x','--pdb', '--pdb-failure'],
+    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                    exit=False)
