@@ -1,13 +1,12 @@
 # pylint: disable-msg=E1101,W0612
-from __future__ import with_statement # for Python 2.5
-from datetime import datetime, time, timedelta, tzinfo
+from datetime import datetime, time, timedelta, tzinfo, date
 import sys
 import os
 import unittest
-
 import nose
 
 import numpy as np
+import pytz
 
 from pandas import (Index, Series, TimeSeries, DataFrame, isnull,
                     date_range, Timestamp)
@@ -19,8 +18,9 @@ import pandas.core.datetools as datetools
 import pandas.tseries.offsets as offsets
 from pandas.tseries.index import bdate_range, date_range
 import pandas.tseries.tools as tools
+from pytz import NonExistentTimeError
 
-from pandas.util.testing import assert_series_equal, assert_almost_equal
+from pandas.util.testing import assert_series_equal, assert_almost_equal, assertRaisesRegexp
 import pandas.util.testing as tm
 
 import pandas.lib as lib
@@ -44,11 +44,12 @@ try:
 except ImportError:
     pass
 
+
 class FixedOffset(tzinfo):
     """Fixed offset in minutes east from UTC."""
 
     def __init__(self, offset, name):
-        self.__offset = timedelta(minutes = offset)
+        self.__offset = timedelta(minutes=offset)
         self.__name = name
 
     def utcoffset(self, dt):
@@ -61,8 +62,11 @@ class FixedOffset(tzinfo):
         return timedelta(0)
 
 fixed_off = FixedOffset(-420, '-07:00')
+fixed_off_no_name = FixedOffset(-330, None)
+
 
 class TestTimeZoneSupport(unittest.TestCase):
+    _multiprocess_can_split_ = True
 
     def setUp(self):
         _skip_if_no_pytz()
@@ -76,6 +80,7 @@ class TestTimeZoneSupport(unittest.TestCase):
 
         self.assert_(rng_eastern.tz == pytz.timezone('US/Eastern'))
 
+
     def test_localize_utc_conversion(self):
         # Localizing to time zone should:
         #  1) check for DST ambiguities
@@ -89,7 +94,8 @@ class TestTimeZoneSupport(unittest.TestCase):
 
         # DST ambiguity, this should fail
         rng = date_range('3/11/2012', '3/12/2012', freq='30T')
-        self.assertRaises(Exception, rng.tz_localize, 'US/Eastern')
+        # Is this really how it should fail??
+        self.assertRaises(NonExistentTimeError, rng.tz_localize, 'US/Eastern')
 
     def test_timestamp_tz_localize(self):
         stamp = Timestamp('3/11/2012 04:00')
@@ -99,6 +105,26 @@ class TestTimeZoneSupport(unittest.TestCase):
         self.assertEquals(result.hour, expected.hour)
         self.assertEquals(result, expected)
 
+    def test_timestamp_constructed_by_date_and_tz(self):
+        """
+        Fix Issue 2993, Timestamp cannot be constructed by datetime.date
+        and tz correctly
+        """
+
+        result = Timestamp(date(2012, 3, 11), tz='US/Eastern')
+
+        expected = Timestamp('3/11/2012', tz='US/Eastern')
+        self.assertEquals(result.hour, expected.hour)
+        self.assertEquals(result, expected)
+
+    def test_timestamp_to_datetime_tzoffset(self):
+        # tzoffset
+        from dateutil.tz import tzoffset
+        tzinfo = tzoffset(None, 7200)
+        expected = Timestamp('3/11/2012 04:00', tz=tzinfo)
+        result = Timestamp(expected.to_datetime())
+        self.assertEquals(expected, result)
+
     def test_timedelta_push_over_dst_boundary(self):
         # #1389
 
@@ -135,7 +161,8 @@ class TestTimeZoneSupport(unittest.TestCase):
 
         dti = DatetimeIndex(start='3/13/2011 1:59', end='3/13/2011 2:00',
                             freq='L')
-        self.assertRaises(pytz.NonExistentTimeError, dti.tz_localize, 'US/Eastern')
+        self.assertRaises(
+            pytz.NonExistentTimeError, dti.tz_localize, 'US/Eastern')
 
     def test_tz_localize_empty_series(self):
         # #2248
@@ -159,7 +186,8 @@ class TestTimeZoneSupport(unittest.TestCase):
         stamp = Timestamp('3/11/2012 05:00', tz='US/Eastern')
         self.assertEquals(stamp.hour, 5)
 
-        rng = date_range('3/11/2012 04:00', periods=10, freq='H', tz='US/Eastern')
+        rng = date_range(
+            '3/11/2012 04:00', periods=10, freq='H', tz='US/Eastern')
 
         self.assertEquals(stamp, rng[1])
 
@@ -180,11 +208,23 @@ class TestTimeZoneSupport(unittest.TestCase):
         rng2 = date_range(start, periods=len(rng), tz=off)
         self.assert_(rng.equals(rng2))
 
-        rng3 = date_range('3/11/2012 05:00:00+07:00', '6/11/2012 05:00:00+07:00')
+        rng3 = date_range(
+            '3/11/2012 05:00:00+07:00', '6/11/2012 05:00:00+07:00')
         self.assert_((rng.values == rng3.values).all())
 
+    def test_create_with_fixedoffset_noname(self):
+        off = fixed_off_no_name
+        start = datetime(2012, 3, 11, 5, 0, 0, tzinfo=off)
+        end = datetime(2012, 6, 11, 5, 0, 0, tzinfo=off)
+        rng = date_range(start=start, end=end)
+        self.assertEqual(off, rng.tz)
+
+        idx = Index([start, end])
+        self.assertEqual(off, idx.tz)
+
     def test_date_range_localize(self):
-        rng = date_range('3/11/2012 03:00', periods=15, freq='H', tz='US/Eastern')
+        rng = date_range(
+            '3/11/2012 03:00', periods=15, freq='H', tz='US/Eastern')
         rng2 = DatetimeIndex(['3/11/2012 03:00', '3/11/2012 04:00'],
                              tz='US/Eastern')
         rng3 = date_range('3/11/2012 03:00', periods=15, freq='H')
@@ -202,7 +242,8 @@ class TestTimeZoneSupport(unittest.TestCase):
         self.assert_(rng[:2].equals(rng2))
 
         # Right before the DST transition
-        rng = date_range('3/11/2012 00:00', periods=2, freq='H', tz='US/Eastern')
+        rng = date_range(
+            '3/11/2012 00:00', periods=2, freq='H', tz='US/Eastern')
         rng2 = DatetimeIndex(['3/11/2012 00:00', '3/11/2012 01:00'],
                              tz='US/Eastern')
         self.assert_(rng.equals(rng2))
@@ -415,6 +456,15 @@ class TestTimeZoneSupport(unittest.TestCase):
         result = to_datetime(dates)
         self.assert_(result.tz == fixed_off)
 
+    def test_fixedtz_topydatetime(self):
+        dates = np.array([datetime(2000, 1, 1, tzinfo=fixed_off),
+                          datetime(2000, 1, 2, tzinfo=fixed_off),
+                          datetime(2000, 1, 3, tzinfo=fixed_off)])
+        result = to_datetime(dates).to_pydatetime()
+        self.assert_(np.array_equal(dates, result))
+        result = to_datetime(dates)._mpl_repr()
+        self.assert_(np.array_equal(dates, result))
+
     def test_convert_tz_aware_datetime_datetime(self):
         # #1581
 
@@ -443,7 +493,7 @@ class TestTimeZoneSupport(unittest.TestCase):
         from dateutil.parser import parse
         from dateutil.tz import tzlocal
         dt = parse('2012-06-13T01:39:00Z')
-        dt = dt.replace(tzinfo = tzlocal())
+        dt = dt.replace(tzinfo=tzlocal())
 
         arr = np.array([dt], dtype=object)
 
@@ -456,14 +506,26 @@ class TestTimeZoneSupport(unittest.TestCase):
         self.assert_(result.tz is pytz.utc)
 
     def test_frame_no_datetime64_dtype(self):
+
         dr = date_range('2011/1/1', '2012/1/1', freq='W-FRI')
         dr_tz = dr.tz_localize('US/Eastern')
         e = DataFrame({'A': 'foo', 'B': dr_tz}, index=dr)
-        self.assert_(e['B'].dtype == object)
+        self.assert_(e['B'].dtype == 'M8[ns]')
+
+        # GH 2810 (with timezones)
+        datetimes_naive   = [ ts.to_pydatetime() for ts in dr ]
+        datetimes_with_tz = [ ts.to_pydatetime() for ts in dr_tz ]
+        df = DataFrame({'dr' : dr, 'dr_tz' : dr_tz,
+                        'datetimes_naive': datetimes_naive,
+                        'datetimes_with_tz' : datetimes_with_tz })
+        result = df.get_dtype_counts()
+        expected = Series({ 'datetime64[ns]' : 3, 'object' : 1 })
+        assert_series_equal(result, expected)
 
     def test_hongkong_tz_convert(self):
         # #1673
-        dr = date_range('2012-01-01','2012-01-10',freq = 'D', tz = 'Hongkong')
+        dr = date_range(
+            '2012-01-01', '2012-01-10', freq='D', tz='Hongkong')
 
         # it works!
         dr.hour
@@ -484,7 +546,8 @@ class TestTimeZoneSupport(unittest.TestCase):
         self.assert_(result.tz == dr_tz.tz)
 
     def test_tz_aware_asfreq(self):
-        dr = date_range('2011-12-01','2012-07-20',freq = 'D', tz = 'US/Eastern')
+        dr = date_range(
+            '2011-12-01', '2012-07-20', freq='D', tz='US/Eastern')
 
         s = Series(np.random.randn(len(dr)), index=dr)
 
@@ -525,11 +588,19 @@ class TestTimeZoneSupport(unittest.TestCase):
 
     def test_frame_from_records_utc(self):
         rec = {'datum': 1.5,
-               'begin_time' : datetime(2006, 4, 27, tzinfo=pytz.utc)}
+               'begin_time': datetime(2006, 4, 27, tzinfo=pytz.utc)}
 
         # it works
         DataFrame.from_records([rec], index='begin_time')
 
+    def test_frame_reset_index(self):
+        dr = date_range('2012-06-02', periods=10, tz='US/Eastern')
+        df = DataFrame(np.random.randn(len(dr)), dr)
+        roundtripped = df.reset_index().set_index('index')
+        xp = df.index.tz
+        rs = roundtripped.index.tz
+        self.assertEquals(xp, rs)
+
     def test_dateutil_tzoffset_support(self):
         from dateutil.tz import tzoffset
         values = [188.5, 328.25]
@@ -540,7 +611,44 @@ class TestTimeZoneSupport(unittest.TestCase):
 
         self.assertEquals(series.index.tz, tzinfo)
 
+        # it works! #2443
+        repr(series.index[0])
+
+    def test_getitem_pydatetime_tz(self):
+        index = date_range(start='2012-12-24 16:00',
+                           end='2012-12-24 18:00', freq='H',
+                           tz='Europe/Berlin')
+        ts = Series(index=index, data=index.hour)
+        time_pandas = Timestamp('2012-12-24 17:00', tz='Europe/Berlin')
+        time_datetime = datetime(2012, 12, 24, 17, 00,
+                                 tzinfo=pytz.timezone('Europe/Berlin'))
+        self.assertEqual(ts[time_pandas], ts[time_datetime])
+
+    def test_index_drop_dont_lose_tz(self):
+        # #2621
+        ind = date_range("2012-12-01", periods=10, tz="utc")
+        ind = ind.drop(ind[-1])
+
+        self.assertTrue(ind.tz is not None)
+
+    def test_datetimeindex_tz(self):
+        """ Test different DatetimeIndex constructions with timezone
+        Follow-up of #4229
+        """
+        
+        arr = ['11/10/2005 08:00:00', '11/10/2005 09:00:00']
+        
+        idx1 = to_datetime(arr).tz_localize('US/Eastern')
+        idx2 = DatetimeIndex(start="2005-11-10 08:00:00", freq='H', periods=2, tz='US/Eastern')
+        idx3 = DatetimeIndex(arr, tz='US/Eastern')
+        idx4 = DatetimeIndex(np.array(arr), tz='US/Eastern')
+        
+        for other in [idx2, idx3, idx4]:
+            self.assert_(idx1.equals(other))
+
+
 class TestTimeZones(unittest.TestCase):
+    _multiprocess_can_split_ = True
 
     def setUp(self):
         _skip_if_no_pytz()
@@ -582,7 +690,7 @@ class TestTimeZones(unittest.TestCase):
         # Can't localize if already tz-aware
         rng = date_range('1/1/2011', periods=100, freq='H', tz='utc')
         ts = Series(1, index=rng)
-        self.assertRaises(Exception, ts.tz_localize, 'US/Eastern')
+        assertRaisesRegexp(TypeError, 'Already tz-aware', ts.tz_localize, 'US/Eastern')
 
     def test_series_frame_tz_convert(self):
         rng = date_range('1/1/2011', periods=200, freq='D',
@@ -606,7 +714,7 @@ class TestTimeZones(unittest.TestCase):
         # can't convert tz-naive
         rng = date_range('1/1/2011', periods=200, freq='D')
         ts = Series(1, index=rng)
-        self.assertRaises(Exception, ts.tz_convert, 'US/Eastern')
+        assertRaisesRegexp(TypeError, "Cannot convert tz-naive", ts.tz_convert, 'US/Eastern')
 
     def test_join_utc_convert(self):
         rng = date_range('1/1/2011', periods=100, freq='H', tz='utc')
@@ -623,7 +731,7 @@ class TestTimeZones(unittest.TestCase):
             self.assert_(isinstance(result, DatetimeIndex))
             self.assert_(result.tz.zone == 'UTC')
 
-    def test_join_naive_with_aware(self):
+    def test_join_aware(self):
         rng = date_range('1/1/2011', periods=10, freq='H')
         ts = Series(np.random.randn(len(rng)), index=rng)
 
@@ -632,6 +740,30 @@ class TestTimeZones(unittest.TestCase):
         self.assertRaises(Exception, ts.__add__, ts_utc)
         self.assertRaises(Exception, ts_utc.__add__, ts)
 
+        test1 = DataFrame(np.zeros((6, 3)),
+                          index=date_range("2012-11-15 00:00:00", periods=6,
+                                           freq="100L", tz="US/Central"))
+        test2 = DataFrame(np.zeros((3, 3)),
+                          index=date_range("2012-11-15 00:00:00", periods=3,
+                                           freq="250L", tz="US/Central"),
+                          columns=range(3, 6))
+
+        result = test1.join(test2, how='outer')
+        ex_index = test1.index.union(test2.index)
+
+        self.assertTrue(result.index.equals(ex_index))
+        self.assertTrue(result.index.tz.zone == 'US/Central')
+
+        # non-overlapping
+        rng = date_range("2012-11-15 00:00:00", periods=6,
+                         freq="H", tz="US/Central")
+
+        rng2 = date_range("2012-11-15 12:00:00", periods=6,
+                          freq="H", tz="US/Eastern")
+
+        result = rng.union(rng2)
+        self.assertTrue(result.tz.zone == 'UTC')
+
     def test_align_aware(self):
         idx1 = date_range('2001', periods=5, freq='H', tz='US/Eastern')
         idx2 = date_range('2001', periods=5, freq='2H', tz='US/Eastern')
@@ -641,6 +773,55 @@ class TestTimeZones(unittest.TestCase):
         self.assertEqual(df1.index.tz, new1.index.tz)
         self.assertEqual(df2.index.tz, new2.index.tz)
 
+    def test_append_aware(self):
+        rng1 = date_range('1/1/2011 01:00', periods=1, freq='H',
+                          tz='US/Eastern')
+        rng2 = date_range('1/1/2011 02:00', periods=1, freq='H',
+                          tz='US/Eastern')
+        ts1 = Series(np.random.randn(len(rng1)), index=rng1)
+        ts2 = Series(np.random.randn(len(rng2)), index=rng2)
+        ts_result = ts1.append(ts2)
+        self.assertEqual(ts_result.index.tz, rng1.tz)
+
+        rng1 = date_range('1/1/2011 01:00', periods=1, freq='H',
+                          tz='UTC')
+        rng2 = date_range('1/1/2011 02:00', periods=1, freq='H',
+                          tz='UTC')
+        ts1 = Series(np.random.randn(len(rng1)), index=rng1)
+        ts2 = Series(np.random.randn(len(rng2)), index=rng2)
+        ts_result = ts1.append(ts2)
+        utc = rng1.tz
+        self.assertEqual(utc, ts_result.index.tz)
+
+        rng1 = date_range('1/1/2011 01:00', periods=1, freq='H',
+                          tz='US/Eastern')
+        rng2 = date_range('1/1/2011 02:00', periods=1, freq='H',
+                          tz='US/Central')
+        ts1 = Series(np.random.randn(len(rng1)), index=rng1)
+        ts2 = Series(np.random.randn(len(rng2)), index=rng2)
+        ts_result = ts1.append(ts2)
+        self.assertEqual(utc, ts_result.index.tz)
+
+    def test_append_aware_naive(self):
+        rng1 = date_range('1/1/2011 01:00', periods=1, freq='H')
+        rng2 = date_range('1/1/2011 02:00', periods=1, freq='H',
+                          tz='US/Eastern')
+        ts1 = Series(np.random.randn(len(rng1)), index=rng1)
+        ts2 = Series(np.random.randn(len(rng2)), index=rng2)
+        ts_result = ts1.append(ts2)
+        self.assert_(ts_result.index.equals(
+            ts1.index.asobject.append(ts2.index.asobject)))
+
+        # mixed
+
+        rng1 = date_range('1/1/2011 01:00', periods=1, freq='H')
+        rng2 = range(100)
+        ts1 = Series(np.random.randn(len(rng1)), index=rng1)
+        ts2 = Series(np.random.randn(len(rng2)), index=rng2)
+        ts_result = ts1.append(ts2)
+        self.assert_(ts_result.index.equals(
+            ts1.index.asobject.append(ts2.index)))
+
     def test_equal_join_ensure_utc(self):
         rng = date_range('1/1/2011', periods=10, freq='H', tz='US/Eastern')
         ts = Series(np.random.randn(len(rng)), index=rng)
@@ -712,6 +893,45 @@ class TestTimeZones(unittest.TestCase):
         rng2 = DatetimeIndex(data=rng, tz='US/Eastern')
         self.assert_(rng.equals(rng2))
 
+    def test_normalize_tz(self):
+        rng = date_range('1/1/2000 9:30', periods=10, freq='D',
+                         tz='US/Eastern')
+
+        result = rng.normalize()
+        expected = date_range('1/1/2000', periods=10, freq='D',
+                              tz='US/Eastern')
+        self.assert_(result.equals(expected))
+
+        self.assert_(result.is_normalized)
+        self.assert_(not rng.is_normalized)
+
+        rng = date_range('1/1/2000 9:30', periods=10, freq='D',
+                         tz='UTC')
+
+        result = rng.normalize()
+        expected = date_range('1/1/2000', periods=10, freq='D',
+                              tz='UTC')
+        self.assert_(result.equals(expected))
+
+        self.assert_(result.is_normalized)
+        self.assert_(not rng.is_normalized)
+
+        from dateutil.tz import tzlocal
+        rng = date_range('1/1/2000 9:30', periods=10, freq='D',
+                         tz=tzlocal())
+        result = rng.normalize()
+        expected = date_range('1/1/2000', periods=10, freq='D',
+                              tz=tzlocal())
+        self.assert_(result.equals(expected))
+
+        self.assert_(result.is_normalized)
+        self.assert_(not rng.is_normalized)
+
+    def test_tzaware_offset(self):
+        dates = date_range('2012-11-01', periods=3, tz='US/Pacific')
+        offset = dates + offsets.Hour(5)
+        self.assertEqual(dates[0] + offsets.Hour(5), offset[0])
+
 if __name__ == '__main__':
-    nose.runmodule(argv=[__file__,'-vvs','-x','--pdb', '--pdb-failure'],
+    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                    exit=False)
