@@ -61,29 +61,32 @@ statement = "df[col][idx]"
 bm_df_getitem2 = Benchmark(statement, setup,
                            name='datamatrix_getitem_scalar')
 
-setup = common_setup + """
-try:
-    klass = DataMatrix
-except:
-    klass = DataFrame
 
+#----------------------------------------------------------------------
+# ix get scalar
+
+setup = common_setup + """
 index = [tm.rands(10) for _ in xrange(1000)]
 columns = [tm.rands(10) for _ in xrange(30)]
-df = klass(np.random.rand(1000, 30), index=index,
+df = DataFrame(np.random.randn(1000, 30), index=index,
                columns=columns)
 idx = index[100]
 col = columns[10]
 """
-statement = "df.get_value(idx, col)"
-bm_df_getitem3 = Benchmark(statement, setup,
-                           name='dataframe_get_value',
-                           start_date=datetime(2011, 11, 12))
+
+indexing_frame_get_value_ix = Benchmark("df.ix[idx,col]", setup,
+                                        name='indexing_frame_get_value_ix',
+                                        start_date=datetime(2011, 11, 12))
+
+indexing_frame_get_value = Benchmark("df.get_value(idx,col)", setup,
+                                     name='indexing_frame_get_value',
+                                     start_date=datetime(2011, 11, 12))
 
 #----------------------------------------------------------------------
 # Boolean DataFrame row selection
 
 setup = common_setup + """
-df = DataFrame(np.random.randn(10000, 4), columns=['A', 'B', 'C', 'D'])
+df  = DataFrame(np.random.randn(10000, 4), columns=['A', 'B', 'C', 'D'])
 indexer = df['B'] > 0
 obj_indexer = indexer.astype('O')
 """
@@ -94,6 +97,36 @@ indexing_dataframe_boolean_rows_object = \
     Benchmark("df[obj_indexer]", setup,
               name='indexing_dataframe_boolean_rows_object')
 
+setup = common_setup + """
+df  = DataFrame(np.random.randn(50000, 100))
+df2 = DataFrame(np.random.randn(50000, 100))
+"""
+indexing_dataframe_boolean = \
+    Benchmark("df > df2", setup, name='indexing_dataframe_boolean',
+              start_date=datetime(2012, 1, 1))
+
+setup = common_setup + """
+import pandas.core.expressions as expr
+df  = DataFrame(np.random.randn(50000, 100))
+df2 = DataFrame(np.random.randn(50000, 100))
+expr.set_numexpr_threads(1)
+"""
+
+indexing_dataframe_boolean_st = \
+    Benchmark("df > df2", setup, name='indexing_dataframe_boolean_st',cleanup="expr.set_numexpr_threads()",
+              start_date=datetime(2013, 2, 26))
+
+
+setup = common_setup + """
+import pandas.core.expressions as expr
+df  = DataFrame(np.random.randn(50000, 100))
+df2 = DataFrame(np.random.randn(50000, 100))
+expr.set_use_numexpr(False)
+"""
+
+indexing_dataframe_boolean_no_ne = \
+    Benchmark("df > df2", setup, name='indexing_dataframe_boolean_no_ne',cleanup="expr.set_use_numexpr(True)",
+              start_date=datetime(2013, 2, 26))
 #----------------------------------------------------------------------
 # MultiIndex sortlevel
 
@@ -118,3 +151,19 @@ inds = range(0, 100, 10)
 
 indexing_panel_subset = Benchmark('p.ix[inds, inds, inds]', setup,
                                   start_date=datetime(2012, 1, 1))
+
+#----------------------------------------------------------------------
+# Iloc
+
+setup = common_setup + """
+df = DataFrame({'A' : [0.1] * 3000, 'B' : [1] * 3000})
+idx = np.array(range(30)) * 99
+df2 = DataFrame({'A' : [0.1] * 1000, 'B' : [1] * 1000})
+df2 = concat([df2, 2*df2, 3*df2])
+"""
+
+frame_iloc_dups = Benchmark('df2.iloc[idx]', setup,
+                            start_date=datetime(2013, 1, 1))
+
+frame_loc_dups = Benchmark('df2.loc[idx]', setup,
+                            start_date=datetime(2013, 1, 1))
